from __future__ import annotations

import logging
from typing import TYPE_CHECKING

from rich.text import Text


if TYPE_CHECKING:
    from llmling_textual.screens.log_screen.log_widget import LoggingWidget


class WidgetHandler(logging.Handler):
    def __init__(self, widget: LoggingWidget):
        super().__init__()
        self.widget = widget

    def emit(self, record: logging.LogRecord) -> None:
        from llmling_textual.screens.log_screen.log_widget import LoggingWidget

        try:
            assert self.widget.formatter
            msg = self.widget.formatter.format(record)
            style = {
                logging.DEBUG: "dim",
                logging.INFO: "white",
                logging.WARNING: "yellow",
                logging.ERROR: "red",
                logging.CRITICAL: "red bold",
            }.get(record.levelno, "white")

            text = Text(msg + "\n", style=style)
            self.widget.post_message(LoggingWidget.LogMessage(text))
        except Exception:  # noqa: BLE001
            self.handleError(record)



from __future__ import annotations

import logging
from typing import ClassVar

from textual.app import App
from textual.binding import Binding

from llmling_agent.delegation.pool import AgentPool
from llmling_textual.screens.log_screen.log_widget import LoggingWidget
from llmling_textual.screens.log_screen.screen import LogWindow
from llmling_textual.screens.main_screen import MainScreen


class AgentApp(App):
    """Main application."""

    DEFAULT_CSS = """
    #main-container {
        height: 100%;
        width: 100%;
    }
    """
    BINDINGS: ClassVar = [
        Binding("f12", "toggle_logs", "Show/Hide Logs"),
    ]

    def __init__(self, config_path: str):
        super().__init__()
        self._config_path = config_path
        self.agent_pool: AgentPool | None = None
        self.main_screen: MainScreen | None = None
        self.log_widget = LoggingWidget()
        logger = logging.getLogger()
        logger.setLevel(logging.DEBUG)
        logger.addHandler(self.log_widget.handler)

    async def on_mount(self) -> None:
        """Initialize pool and mount main screen."""
        self.agent_pool = AgentPool(self._config_path)
        await self.agent_pool.__aenter__()

        self.main_screen = MainScreen(self.agent_pool)
        await self.push_screen(self.main_screen)

    async def on_unmount(self) -> None:
        """Clean up resources."""
        if self.main_screen:
            # First disconnect event handlers
            self.main_screen.cleanup()

        if self.agent_pool:
            await self.agent_pool.__aexit__(None, None, None)

    def action_toggle_logs(self) -> None:
        """Toggle log window."""
        if isinstance(self.screen, LogWindow):
            self.pop_screen()
        else:
            self.push_screen(LogWindow(self.log_widget))


if __name__ == "__main__":
    app = AgentApp("src/llmling_agent/config_resources/agents.yml")
    app.run()



from __future__ import annotations

import logging
from typing import TYPE_CHECKING

from textual.message import Message
from textual.widgets import RichLog

from llmling_textual.screens.log_screen.widget_handler import WidgetHandler


if TYPE_CHECKING:
    from rich.text import Text


class LoggingWidget(RichLog):
    """Widget that acts as both log display and handler."""

    class LogMessage(Message):
        """Internal message for log events."""

        def __init__(self, text: Text):
            self.text = text
            super().__init__()

    DEFAULT_CSS = """
    LoggingWidget {
        height: 1fr;
        border: heavy $background;
        padding: 1;
    }
    """

    def __init__(
        self,
        *,
        level: int = logging.DEBUG,
        format_string: str = "%(asctime)s - %(levelname)s - %(message)s",
        id: str | None = None,  # noqa: A002
    ):
        super().__init__(id=id, wrap=True, markup=True)
        self.formatter = logging.Formatter(format_string)
        handler = WidgetHandler(self)
        handler.setLevel(level)
        self.handler = handler

    def on_logging_widget_log_message(self, message: LogMessage) -> None:
        """Handle log message event."""
        self.write(message.text)
